/*
 * Copyright 2017 Red Hat, Inc. and/or its affiliates.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *   http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.kie.workbench.common.dmn.api.definition.v1_1;

import java.util.HashSet;
import java.util.Set;

import org.jboss.errai.common.client.api.annotations.MapsTo;
import org.jboss.errai.common.client.api.annotations.NonPortable;
import org.jboss.errai.common.client.api.annotations.Portable;
import org.jboss.errai.databinding.client.api.Bindable;
import org.kie.workbench.common.dmn.api.property.dmn.Id;
import org.kie.workbench.common.dmn.api.rules.AcyclicDirectedGraphRule;
import org.kie.workbench.common.dmn.api.rules.SingleConnectorPerTypeGraphRule;
import org.kie.workbench.common.dmn.api.validation.NoValidation;
import org.kie.workbench.common.forms.adf.definitions.annotations.FormDefinition;
import org.kie.workbench.common.forms.adf.definitions.settings.FieldPolicy;
import org.kie.workbench.common.stunner.core.definition.annotation.Definition;
import org.kie.workbench.common.stunner.core.definition.annotation.Description;
import org.kie.workbench.common.stunner.core.definition.annotation.definition.Category;
import org.kie.workbench.common.stunner.core.definition.annotation.definition.Labels;
import org.kie.workbench.common.stunner.core.definition.annotation.definition.Title;
import org.kie.workbench.common.stunner.core.factory.graph.EdgeFactory;
import org.kie.workbench.common.stunner.core.rule.annotation.CanConnect;
import org.kie.workbench.common.stunner.core.rule.annotation.RuleExtension;

@Portable
@Bindable
@Definition(graphFactory = EdgeFactory.class, builder = Association.AssociationBuilder.class)
@FormDefinition(policy = FieldPolicy.ONLY_MARKED, startElement = "id")
@CanConnect(startRole = "business-knowledge-model", endRole = "text-annotation")
@CanConnect(startRole = "decision", endRole = "text-annotation")
@CanConnect(startRole = "input-data", endRole = "text-annotation")
@CanConnect(startRole = "knowledge-source", endRole = "text-annotation")
@RuleExtension(handler = AcyclicDirectedGraphRule.class, typeArguments = {Association.class})
@RuleExtension(handler = SingleConnectorPerTypeGraphRule.class, typeArguments = {Association.class})
@NoValidation
public class Association extends Artifact {

    @Category
    public static final transient String stunnerCategory = Categories.CONNECTORS;

    @Title
    public static final transient String stunnerTitle = "DMN Association";

    @Description
    public static final transient String stunnerDescription = "DMN Association";

    @Labels
    private final Set<String> stunnerLabels = new HashSet<String>() {{
        add("association");
    }};

    @NonPortable
    public static class AssociationBuilder extends BaseNodeBuilder<Association> {

        @Override
        public Association build() {
            return new Association(new Id(),
                                   new org.kie.workbench.common.dmn.api.property.dmn.Description());
        }
    }

    public Association() {
    }

    public Association(final @MapsTo("id") Id id,
                       final @MapsTo("description") org.kie.workbench.common.dmn.api.property.dmn.Description description) {
        super(id,
              description);
    }

    // -----------------------
    // Stunner core properties
    // -----------------------

    public String getStunnerCategory() {
        return stunnerCategory;
    }

    public String getStunnerTitle() {
        return stunnerTitle;
    }

    public String getStunnerDescription() {
        return stunnerDescription;
    }

    public Set<String> getStunnerLabels() {
        return stunnerLabels;
    }
}
